const passport = require('passport');
const LocalStrategy = require('passport-local').Strategy;
const accounts=require('../account/Accounts'); // 인증 데이터가 담긴 객체 반환 (세션 시크릿 포함)

module.exports=function(router){
  router.use(passport.initialize()); // passport에 express에 개입
  router.use(passport.session()); // passport를 사용하면서 session을 내부적으로 사용

  // serializeUser의 콜백은 로그인이 성공했을 때 호출됨
  // authData가 user인수에 들어옴 (LocalStrategy 미들웨어의 done함수가 진행)
  passport.serializeUser(function(user, done) {
    done(null, user.nickname); // -> user.nickname이 세션 데이터 안 passport의 user값으로 전달
    // serializeUser: < 로그인에 성공했다는 사실을 세션 스토어에 저장하는 기능 >
  });

  // deserializeUser의 콜백은 로그인 성공 후 페이지를 접속할 때마다 호출됨
  // deserializeUser: < 사용자 정보를 세션 스토어에서 불러오는 기능 >
  // 콜백의 user인수는 세션 스토어의 passposrt에 담긴 user임
  passport.deserializeUser(function(user, done) {
    done(null,user); // user는 request.user속성으로 저장됨
  });

  // 서버에 저장된 인증 데이터 (현재: 첫 번째 계정 객체)
  const authData=accounts.a1;

  // Local방식을 처리하는 미들웨어 [LocalStrategy 미들웨어]
  passport.use(
      new LocalStrategy(
        {
          usernameField: 'id', // login미들웨어에서 사용하는 username 변수:id
          passwordField: 'password' // login미들웨어에서 사용하는 password 변수:password
        },
        // 사용자가 입력한 계정 정보와 비교
        function (username, password, done) {
          if(username === authData.id){
             // 로그인 성공 -> serializeUser콜백 호출
             if(password === authData.password)
               return done(null, authData,{});
             // 로그인 실패(비밀번호 오류) -> err_code 업데이트
             else {
               accounts.err_code=1;
               return done(null, false,{});
             }
         }
         // 로그인 실패(아이디 오류) -> err_code 업데이트
         else{
           accounts.err_code=2;
           return done(null, false);
        }
      }
    )
  );
  return passport;
}
