const express=require('express'); // 함수를 반환
const router=express.Router(); // app객체처럼 router객체를 반환
const fs = require('fs');
const qs=require('querystring');
const path=require('path');
const accounts=require('../account/Accounts'); // 인증 데이터가 담긴 객체 반환 (세션 시크릿 포함)
const Template=require('../lib/Template');
const sanitizeHtml=require('sanitize-html'); // 함수를 반환

// 클라이언트에서 입력한 데이터를 처리하는 경우 <글 생성>
// 사용자가 create-submit을 통해 서버에 데이터를 전송했을 때 (응답 코드)
router.post('/process_create',(request,response)=>{
  if(request.user===undefined){
    response.send(Template.LOGIN());
    return false;
  }
    let post = request.body;
    let title = post.title.replace(/(\s*)/g, "");
    let description = post.description;
    let filteredTitle=path.parse(title).base;
    fs.writeFile(`data/${filteredTitle}`, description, 'utf8', function(err){
      response.redirect(`/page/${filteredTitle}`);
    });
});


// 클라이언트에서 입력한 데이터를 처리하는 경우 <글 수정>
// 사용자가 update-submit을 통해 서버에 데이터를 전송했을 때 (응답 코드)
router.post('/process_update',(request,response)=>{
    if(request.user===undefined){
      response.send(Template.LOGIN());
      return false;
    }
    let post = request.body;
    let id=post.id;
    let title = post.title.replace(/(\s*)/g, "");
    let description = post.description;
    let filteredTitle=path.parse(title).base;
    fs.rename(`data/${id}`, `data/${filteredTitle}`, function(err){
      fs.writeFile(`data/${filteredTitle}`, description, 'utf8', function(err){
        response.redirect(`/page/${filteredTitle}`);
      })
    })
});

// 클라이언트에서 입력했던 데이터를 처리하는 경우 <글 삭제>
// 사용자가 delete버튼을 통해 서버에 데이터를 전송했을 때 (응답 코드
router.post('/process_delete',(request,response)=>{
    if(request.user===undefined){
      response.send(Template.LOGIN());
      return false;
    }
    let post = request.body;
    let id=post.id;
    let filteredId=path.parse(id).base;
    fs.unlink(`data/${filteredId}`, function(err){
        response.redirect('/');
    })
});

// 클라이언트에서 입력한 데이터를 처리하지 않는 경우 (일반적: login 페이지)
router.get('/login',(request,response)=>{
    // 이미 로그인된 경우
    if(request.user!==undefined){
      response.send('You are logged in !!');
      return false;
    }
    let login_status='';
    // err_code에 따라 다른 플래시 메시지 생성
    if(accounts.err_code > 0){
      request.flash(); // 기존 플래시 메시지 삭제
      if(accounts.err_code===1){ request.flash('error', 'Incorrect Password !!');}
      else{ request.flash('error', 'Incorrect Id !!');}
	  accounts.err_code=0;
      login_status=request.flash().error[0];
    }
    let title="Login";
    let li_list=Template.LIST(request.list);
    let template=Template.HTML(title,Template.AUTH(request.user),li_list,
      `<form action="/log_in_out/process_login" method="post">
        <p><input type="text" name="id" placeholder="id"></p>
        <p><input type="password" name="password" placeholder="password"></p>
        <p style="color:red;">${login_status}</p>
        <p><input type="submit"></p>
      </form>`
      ,'');
    response.send(template);
});

// 클라이언트에서 입력한 데이터를 처리하지 않는 경우 (일반적: create 페이지)
// 이 소스는 '/:pageId'라우트보다 먼저 와야 된다. (이유: 순차처리로, data디렉토리에 create파일이 없어서 next(err)이 호출되기 때문)
router.get('/create',(request,response)=>{
    if(request.user===undefined){
      response.send(Template.LOGIN());
      return false;
    }
    let title="Create";
    let li_list=Template.LIST(request.list);
    let template=Template.HTML(title,Template.AUTH(request.user),li_list,Template.FORM('/page/process_create',`value=""`,""),'');
    response.send(template);
});

// 클라이언트에서 입력한 데이터를 처리하지 않는 경우 (일반적: page/:pageId 페이지 <포스팅> )
router.get('/:pageId',(request,response,next)=>{
    let filteredId=path.parse(request.params.pageId).base;
    fs.readFile(`data/${filteredId}`,'utf8',(err,description)=>{
      // 디렉토리에 해당하는 파일이 없으면 err값은 true가 됨
      if(err){
        next(err); // 에러 핸들링하는 미들웨어 호출
      }
      else{
        let sanitizedTitle=sanitizeHtml(request.params.pageId);
        let sanitizeDescription=sanitizeHtml(description,{allowedTags:['h1']});
        let li_list=Template.LIST(request.list);
        let template=Template.HTML(sanitizedTitle,Template.AUTH(request.user),li_list,sanitizeDescription,`
          <!-- 글 수정 버튼 -->
          <a href="/page/update/${sanitizedTitle}">update</a>

          <!-- 글 삭제 버튼 -->
          <form action="/page/process_delete" method="post">
            <input type="hidden" name="id" value="${sanitizedTitle}">
            <input type="submit" value="delete">
          </form>
          `);
        response.send(template);
        }
    });
});


// 클라이언트에서 입력한 데이터를 처리하지 않는 경우 (일반적: update/:pageId 페이지)
router.get('/update/:pageId',(request,response)=>{
    if(request.user===undefined){
      response.send(Template.LOGIN());
      return false;
    }
    let filteredId=path.parse(request.params.pageId).base;
    fs.readFile(`data/${filteredId}`,'utf8',(err,description)=>{
        let sanitizedTitle=sanitizeHtml(request.params.pageId);
        let sanitizeDescription=sanitizeHtml(description,{allowedTags:['h1']});
        let li_list=Template.LIST(request.list);
        let template=Template.HTML(sanitizedTitle,Template.AUTH(request.user),li_list,Template.FORM('/page/process_update',`value=${sanitizedTitle}`,sanitizeDescription),'');
        response.send(template);
    });
});

// 외부 모듈에 연결
module.exports=router;
